package DE_EPAGES::Cartridge::t::XML; ## no critic qw(RequireCommentForExportedFunctions)
use base DE_EPAGES::Database::API::Test;

use Test::Simple tests => 20;

use strict;
use Text::Diff qw( diff );
use DE_EPAGES::Core::API::Error qw ( GetError ExistsError Error );
use DE_EPAGES::Core::API::File qw ( normalizeXML DeleteFile );
use DE_EPAGES::Cartridge::API::Constants qw( STATE_INSTALLED );
use DE_EPAGES::Cartridge::API::XML::CartridgesImportHandler qw ( ImportCartridges );
use DE_EPAGES::Cartridge::API::XML::CartridgesDeleteHandler qw ( DeleteCartridges );
use DE_EPAGES::Cartridge::API::Table::Cartridge qw (InsertCartridge GetPKeyCartridgesByPackage GetPKeyCartridgeByPackage DeleteCartridge ExistsCartridgeByPackage);
use DE_EPAGES::Cartridge::API::Table::CartridgeDependency qw (ExistsCartridgeDependency GetInfoCartridgeDependenciesByDependendCartridgeID);

sub test {

    ok ( !ExistsCartridgeByPackage($_), "$_ does not yet exist" ) foreach qw(TestCartridge1 TestCartridge2 TestCartridge3 TestCartridge4);

    ImportCartridges('01_s.xml');
    ok ( ExistsCartridgeByPackage($_), "$_ exists" ) foreach qw(TestCartridge1 TestCartridge2 TestCartridge3);
    ok ( !ExistsCartridgeByPackage('TestCartridge4'), 'TestCartridge4 does not exist' );

    my $CartidgeID1 = GetPKeyCartridgeByPackage('TestCartridge1');
    my $CartidgeID2 = GetPKeyCartridgeByPackage('TestCartridge2');

    ok( ExistsCartridgeDependency( $CartidgeID1, $CartidgeID2 ), 'ExistsCartridgeDependency' );
    ok( !ExistsCartridgeDependency( $CartidgeID2, $CartidgeID1 ), 'not ExistsCartridgeDependency' );
    my $aDependencies = GetInfoCartridgeDependenciesByDependendCartridgeID($CartidgeID2);
    ok( 1 == scalar @$aDependencies, 'count dependencies' );
    ok( $CartidgeID1 == $aDependencies->[0]->{'CartridgeID'}, 'GetInfoCartridgeDependenciesByDependendCartridgeID' );

    ImportCartridges('01_s.xml'); # update

    Test_Export();
    Test_ExportFalse();
    Test_ImportDeleteCartridgeDependency();

    DeleteCartridges('01_s.xml');
    ok ( !ExistsCartridgeByPackage($_), "$_ deleted" ) foreach qw(TestCartridge1 TestCartridge2 TestCartridge3);
}

sub Test_Export {

    TestDriver->new->exportXML('01_e.xml',{'encoding' => 'iso-8859-1'});

    my $Expected = normalizeXML('01_s.xml');
    my $Found = normalizeXML('01_e.xml');
    ok($Found eq $Expected, 'exported file equals reference file');
}

sub Test_ExportFalse {

    InsertCartridge({
        Package =>'TestCartridge4',
        Version =>'1.4',
        DBStructureVersion =>'1.1',
        InstallState=> STATE_INSTALLED
    });
    TestDriver->new->exportXML('01_f.xml',{'encoding' => 'iso-8859-1'});

    my $Expected = normalizeXML('01_s.xml');
    my $Found = normalizeXML('01_f.xml');
    ok($Found ne $Expected, 'exported file differs from reference file');
}

sub Test_ImportDeleteCartridgeDependency {

    my $CartridgeID = GetPKeyCartridgeByPackage('TestCartridge2');
    my $DependendCartridgeID = GetPKeyCartridgeByPackage('TestCartridge3');
    ok( !ExistsCartridgeDependency( $CartridgeID, $DependendCartridgeID ), 'before import XML_CartridgeDependency');

    ImportCartridges('XML_CartridgeDependency.xml');
    ok( ExistsCartridgeDependency( $CartridgeID, $DependendCartridgeID ), 'after import XML_CartridgeDependency');

    DeleteCartridges('XML_CartridgeDependency.xml');
    ok( !ExistsCartridgeDependency( $CartridgeID, $DependendCartridgeID ), 'after delete XML_CartridgeDependency');
}

sub cleanup {

    DeleteCartridges('01_s.xml');
    DeleteCartridge($_) foreach @{GetPKeyCartridgesByPackage('TestCartridge1')};
    DeleteCartridge($_) foreach @{GetPKeyCartridgesByPackage('TestCartridge2')};
    DeleteCartridge($_) foreach @{GetPKeyCartridgesByPackage('TestCartridge3')};
    DeleteCartridge($_) foreach @{GetPKeyCartridgesByPackage('TestCartridge4')};
    DeleteFile('01_e.xml');
    DeleteFile('01_f.xml');
}

__PACKAGE__->runScript;

package TestDriver; ## no critic qw(ProhibitMultiplePackages)
use base qw( DE_EPAGES::Cartridge::API::XML::CartridgesExportDriver );

use strict;
use DE_EPAGES::Cartridge::API::Table::Cartridge qw( GetPKeyCartridgesByPackage );##no critic (RequireCorrectImports)

sub AllCartridges {
    my $self = shift;
    my $IDs = GetPKeyCartridgesByPackage('TestCartridge1');
    $self->addElement( 'Cartridge', { 'CartridgeID' => $_ } ) foreach @$IDs;
    $IDs = GetPKeyCartridgesByPackage('TestCartridge2');
    $self->addElement( 'Cartridge', { 'CartridgeID' => $_ } ) foreach @$IDs;
    $IDs = GetPKeyCartridgesByPackage('TestCartridge3');
    $self->addElement( 'Cartridge', { 'CartridgeID' => $_ } ) foreach @$IDs;
    $IDs = GetPKeyCartridgesByPackage('TestCartridge4');
    $self->addElement( 'Cartridge', { 'CartridgeID' => $_ } ) foreach @$IDs;
}

# get attributes for the document tag
sub epages {
    return {};
}

sub drive {
    my $self = shift;

#    $self->addHook( 'epages' => 'AllCartridges' );
    $self->addCartridgesHooks;

    $self->addElement( 'epages' );
}

1;

