#==============================================================================
# package      DE_EPAGES::Core::API::Constants
# state        public
#------------------------------------------------------------------------------
# description  Some constants
#==============================================================================
package DE_EPAGES::Core::API::Constants;
use base qw( Exporter );

use strict;

## no critic qw(ProhibitAutomaticExportationNonDAL)
our @EXPORT = qw(
    PERL516
    PERL538
    DEFAULT_ENCODING
    TEXT_FILE_TYPES
    TEXT_FILE_NAMES
    IMAGE_FILE_TYPES
    PDF_FILE_TYPES

    SECONDS_PER_DAY
    SECONDS_PER_HOUR
    SECONDS_PER_MINUTE
    MINUTES_PER_HOUR

    OPEN_MODI
    EPAGES_IP

    USER_R
    USER_RX
    USER_RW
    USER_RWX
    ALL_X
    ALL_R
    ALL_RX
    ALL_RW
    ALL_RWX
    ALL_RWS
    GROUP_R
    GROUP_RX
    OWNER_RW
    OWNER_RWX
    BASE_RW
    BASE_RWX
    BASE_MASK
    SET_EFFECTIVE

    MICRO
    MILLI
    CENTI
    DECI
    DECA
    HECTO
    KILO
    MEGA
    GIGA
    TERA
    PETA
    EXA
    ZETA
    YOTTA
    KIBI
    MEBI
    GIBI
    TEBI
    PEBI
    EXBI
    ZEBI
    YOBI

    LOG_EMERG
    LOG_ALERT
    LOG_CRIT
    LOG_ERR
    LOG_WARNING
    LOG_NOTICE
    LOG_INFO
    LOG_DEBUG

    LOG_KERN
    LOG_USER
    LOG_MAIL
    LOG_DAEMON
    LOG_AUTH
    LOG_SYSLOG
    LOG_LPR
    LOG_NEWS
    LOG_UUCP
    LOG_CRON
    LOG_AUTHPRIV
    LOG_FTP

    %SYSLOGLEVELS
    SYSLOCFACILITY

    CALLER_PACKAGE
    CALLER_FILENAME
    CALLER_LINE
    CALLER_SUBNAME
    CALLER_HASARGS
    CALLER_WANTARRAY
    CALLER_EVALTEXT
    CALLER_ISREQUIRE

    PASSWORD_DUMMY

    STATISTIC_POSITION_COUNT
    STATISTIC_POSITION_TIME
    STATISTIC_POSITION_TIMES

    BOM

    IMAGE_MAGICK_RESIZE_FILTER
    IMAGE_MAGICK_RESIZE_BLUR
    IMAGE_MAGICK_WARNING
    MAX_IMAGE_MEGAPIXELS
    DEFAULT_JPEG_QUALITY

    TRANSFORMATION_TYPES

    INLINE_DIR
    INLINE_JAVA_SERVER_PORT
    INLINE_STARTUP_DELAY

    REST_DEFAULT_PROTOCOL
    REST_DEFAULT_DOMAIN
    REST_DEFAULT_PORT
    REST_DEFAULT

    PDF_SERVICE_REQUEST_HEADER

    ENCODINGS_LT
    ENCODINGS_GT

    TAG_URI_ATTRIBUTE

    DEFAULT_DENSITY
    INCH_IN_CM
    DEFAULT_STRINGCOMPARE_COVERAGE
);

use vars qw (
              %SYSLOGLEVELS
            );

#========================================================================================
# global       PERL516 | perl version number 5.16.3
#========================================================================================
use constant PERL516 => 5.016003;

#========================================================================================
# global       PERL538 | perl version number 5.38.0
#========================================================================================
use constant PERL538 => 5.038000;

#========================================================================================
# global       DEFAULT_ENCODING | default encoding for text files: 'iso-8859-1' | string
#========================================================================================
use constant DEFAULT_ENCODING => 'iso-8859-1';

## no critic qw(RequireCommentForExportedFunctions)

#========================================================================================
# global       TEXT_FILE_TYPES | known text file extensions used by IsTextFile function
#               (cgi cmd conf css csv dtd htm html ini js json php pl sh smd sh sql svd
#               t txt wsdl xml xsd xsl sh,cron)
#               Use lowercase extensions here!
#               | array.string
#========================================================================================
sub TEXT_FILE_TYPES () { return qw ( as cgi cmd conf css csv dtd htm html ini js json php pl smd sh sql svg t template txt wsdl xml xsd xsl ), 'sh,cron'; }


#========================================================================================
# global       TEXT_FILE_NAMES | known text file Names used by IsTextFile function
#               (LICENSE README)
#               | array.string
#========================================================================================
sub TEXT_FILE_NAMES () { return qw ( LICENSE README TODO); }


#========================================================================================
# global       IMAGE_FILE_TYPES | known image file extensions used by IsImageFile function
#               (bmp jpeg jpg gif png tif wbmp)
#               Use lowercase extensions here!
#               | array.string
#========================================================================================
sub IMAGE_FILE_TYPES () { return qw ( bmp jpeg jpg gif png tif wbmp ); }

#========================================================================================
# global       PDF_FILE_TYPES | known pdf file extensions used by IsPDFFile function
#               (pdf)
#               Use lowercase extensions here!
#               | array.string
#========================================================================================
sub PDF_FILE_TYPES () { return qw ( pdf ); }


#========================================================================================
# global       OPEN_MODI | open modi for text files (eg: log files) | ref.hash
#========================================================================================
sub OPEN_MODI () {
    return {
        'append' => '>>',
        'write'  => '>',
        'read'   => '<',
    };
}

#========================================================================================
# global       EPAGES_IP | ePages external IP list | ref.array.string
#========================================================================================
sub EPAGES_IP () {
    ## no critic qw(ProhibitMagicNumbers)
    my %IPv4List = ( '91.216.90'   => [1..254],
                     '213.128.154' => [1..63],
                     '213.128.154' => [129..190],
                     '217.17.192'  => [1..254],
                     '217.17.193'  => [66..79],
                     '217.17.196'  => [1..63],
        );
    ## use critic qw(ProhibitMagicNumbers)
    my @IP;
    foreach my $IP (keys %IPv4List) {
        push @IP, "$IP.$_" foreach @{$IPv4List{$IP}};
    }
    return \@IP;
}

#========================================================================================
#time constants
use constant SECONDS_PER_DAY => 24 * 60 * 60;
use constant SECONDS_PER_HOUR => 60 ** 2;
use constant SECONDS_PER_MINUTE => 60;
use constant MINUTES_PER_HOUR => 60;

#========================================================================================
#constants for file modes

##no critic qw(ProhibitLeadingZeros)
use constant USER_R    => 0400;  # u=r
use constant USER_RX   => 0500;  # u=rx
use constant USER_RW   => 0600;  # u=rw
use constant USER_RWX  => 0700;  # u=rwx

use constant ALL_X     => 0111;  # a=x
use constant ALL_R     => 0444;  # a=r
use constant ALL_RX    => 0555;  # a=rx
use constant ALL_RW    => 0666;  # a=rx
use constant ALL_RWX   => 0777;  # a=rwx
use constant ALL_RWS   => 07777; # a=rws

use constant GROUP_R   => 0640;  # u=rw,g=r
use constant GROUP_RX  => 0750;  # u=rwx,g=rx

use constant OWNER_RW  => 0660;  # ug=rw
use constant OWNER_RWX => 0770;  # ug=rwx

use constant BASE_RW   => 0644;  # u=rw,go=r
use constant BASE_RWX  => 0755;  # u=rwx,go=rx

use constant BASE_MASK => 022;   # u=rwx,go=rx
use constant SET_EFFECTIVE => 07000; # a=s
#use critic qw(ProhibitLeadingZeros)

# OWNER_RW|ALL_R                => 0664
# OWNER_RWX|ALL_RX              => 0775
# ALL_RW|SET_EFFECTIVE          => 07666
# USER_RWX|ALL_RX|SET_EFFECTIVE => 07755

#========================================================================================
#constants for unit prefixes (see http://en.wikipedia.org/wiki/Byte)

##no critic qw(RequireCorrectImports)
use constant MICRO => 10 ** -6;
use constant MILLI => 10 ** -3;
use constant CENTI => 10 ** -2;
use constant DECI  => 10 ** -1;
use constant DECA  => 10 ** 1;
use constant HECTO => 10 ** 2;
use constant KILO  => 10 ** 3;
use constant MEGA  => KILO ** 2;
use constant GIGA  => KILO ** 3;
use constant TERA  => KILO ** 4;
use constant PETA  => KILO ** 5;
use constant EXA   => KILO ** 6;
use constant ZETA  => KILO ** 7;
use constant YOTTA => KILO ** 8;

use constant KIBI => 2 ** 10;
use constant MEBI => KIBI ** 2;
use constant GIBI => KIBI ** 3;
use constant TEBI => KIBI ** 4;
use constant PEBI => KIBI ** 5;
use constant EXBI => KIBI ** 6;
use constant ZEBI => KIBI ** 7;
use constant YOBI => KIBI ** 8;
##use critic qw(RequireCorrectImports)
## no critic qw(RequireCommentForExportedFunctions)

#========================================================================================
#constants for syslogappender
# ---- levels
use constant    LOG_EMERG   => 0    ; # system is unusable
use constant    LOG_ALERT   => 1    ; # action must be taken immediately
use constant    LOG_CRIT    => 2    ; # critical conditions
use constant    LOG_ERR     => 3    ; # error conditions
use constant    LOG_WARNING => 4    ; # warning conditions
use constant    LOG_NOTICE  => 5    ; # normal but significant condition
use constant    LOG_INFO    => 6    ; # informational
use constant    LOG_DEBUG   => 7    ; # debug-level messages

# ---- facilitiy
use constant    LOG_KERN    => (0<<3)   ; # kernel messages
use constant    LOG_USER    => (1<<3)   ; # random user-level messages
use constant    LOG_MAIL    => (2<<3)   ; # mail system
use constant    LOG_DAEMON  => (3<<3)   ; # system daemons
use constant    LOG_AUTH    => (4<<3)   ; # security/authorization messages
use constant    LOG_SYSLOG  => (5<<3)   ; # messages generated internally by syslogd
use constant    LOG_LPR     => (6<<3)   ; # line printer subsystem
use constant    LOG_NEWS    => (7<<3)   ; # network news subsystem
use constant    LOG_UUCP    => (8<<3)   ; # UUCP subsystem
use constant    LOG_CRON    => (9<<3)   ; # clock daemon
use constant    LOG_AUTHPRIV    => (10<<3)  ; # security/authorization messages (private)
use constant    LOG_FTP     => (11<<3)  ; # ftp daemon

use constant    LOG_LOCAL0  => (16<<3)  ; # reserved for local use
use constant    LOG_LOCAL1  => (17<<3)  ; # reserved for local use
use constant    LOG_LOCAL2  => (18<<3)  ; # reserved for local use
use constant    LOG_LOCAL3  => (19<<3)  ; # reserved for local use
use constant    LOG_LOCAL4  => (20<<3)  ; # reserved for local use
use constant    LOG_LOCAL5  => (21<<3)  ; # reserved for local use
use constant    LOG_LOCAL6  => (22<<3)  ; # reserved for local use
use constant    LOG_LOCAL7  => (23<<3)  ; # reserved for local use

# mapping from epages error levels to syslog levels
%SYSLOGLEVELS = (
                    INFO    => LOG_INFO,
                    DEBUG   => LOG_DEBUG,
                    WARN    => LOG_WARNING,
                    ERROR   => LOG_ERR,
                    FATAL   => LOG_CRIT,
                );

use constant    SYSLOCFACILITY => LOG_USER;


use constant CALLER_PACKAGE   => 0;
use constant CALLER_FILENAME  => 1;
use constant CALLER_LINE      => 2;
use constant CALLER_SUBNAME   => 3;
use constant CALLER_HASARGS   => 4;
use constant CALLER_WANTARRAY => 5;
use constant CALLER_EVALTEXT  => 6;
use constant CALLER_ISREQUIRE => 7;

#========================================================================================
# global       PASSWORD_DUMMY | the dummy text which is displayed in password fields
#                   (********) | string
#========================================================================================

use constant PASSWORD_DUMMY => '********';

#========================================================================================
# global       STATISTIC_POSITION_COUNT | position of count in statistic array | 0
# global       STATISTIC_POSITION_TIME | position of time in statistic array | 1
# global       STATISTIC_POSITION_TIMES | position of times (system,user) in statistic array | 2
#========================================================================================
use constant STATISTIC_POSITION_COUNT => 0;
use constant STATISTIC_POSITION_TIME  => 1;
use constant STATISTIC_POSITION_TIMES => 2;

#========================================================================================
# global       BOM | Unicode Byte Order Mark = chr(0xFEFF) | string
#========================================================================================
use constant BOM => chr(0xFEFF);

#========================================================================================
# global       IMAGE_MAGICK_RESIZE_FILTER | Filter to use when resizing images | string
# global       IMAGE_MAGICK_RESIZE_BLUR | Blur to use when resizing images | float
#========================================================================================
use constant IMAGE_MAGICK_RESIZE_FILTER => 'Cubic';
use constant IMAGE_MAGICK_RESIZE_BLUR => 0.85;

#========================================================================================
# global       MAX_IMAGE_MEGAPIXELS | Maximal size of an uploaded image (width * height) | integer
#========================================================================================
use constant MAX_IMAGE_MEGAPIXELS => 20;

#========================================================================================
# global       DEFAULT_JPEG_QUALITY | Default quality value for JPEG images, 0 - 100 | integer
#========================================================================================
use constant DEFAULT_JPEG_QUALITY => 85;

#========================================================================================
# global       TRANSFORMATION_TYPES | Transformation types use for image transformations
#               | array.string
#========================================================================================
sub TRANSFORMATION_TYPES () { return qw ( Color Image Object ); }

#========================================================================================
# global       REST_DEFAULT_PROTOCOL | default value for rest protocol | string
# global       REST_DEFAULT_DOMAIN | default value for rest domain name | string
# global       REST_DEFAULT_PORT | default value for rest server port | integer
#========================================================================================
use constant REST_DEFAULT_PROTOCOL => 'http';
use constant REST_DEFAULT_DOMAIN => 'localhost';
use constant REST_DEFAULT_PORT => 8088;
use constant REST_DEFAULT => 'http://localhost:8088/rs/';

#========================================================================================
# global       PDF_SERVICE_REQUEST_HEADER | content type for post request of pdf service | ref.hash
#========================================================================================
use constant PDF_SERVICE_REQUEST_HEADER => { 'Content_Type' => 'text/html' };

#========================================================================================
# global       ENCODINGS_LT | all encodings for "less then" char | ref.array
#========================================================================================
use constant ENCODINGS_LT => [
    '<',
    '%3C',
    '&lt',

    '&#60',
    '&#060',
    '&#0060',
    '&#00060',
    '&#000060',
    '&#0000060',

    '&#x3c',
    '&#x03c',
    '&#x003c',
    '&#x0003c',
    '&#x00003c',
    '&#x000003c',

    '\x{003C}' ##no critic qw(RequireInterpolationOfMetachars)
];

#========================================================================================
# global       ENCODINGS_GT | all encodings for "greater then" char | ref.array
#========================================================================================
use constant ENCODINGS_GT => [
    '>',
    '%3e',
    '&gt',

    '&#62',
    '&#062',
    '&#0062',
    '&#00062',
    '&#000062',
    '&#0000062',

    '&#x3e',
    '&#x03e',
    '&#x003e',
    '&#x0003e',
    '&#x00003e',
    '&#x000003e',

    '\x{003E}' ##no critic qw(RequireInterpolationOfMetachars)
];

#========================================================================================
# global       TAG_URI_ATTRIBUTE | key is tag, value of tag-attribute (e.g. 'a' => 'href')| ref.hash
#========================================================================================
use constant TAG_URI_ATTRIBUTE => {
    'a'      => 'href',
    'link'   => 'href',
    'audio'  => 'src',
    'embed'  => 'src',
    'iframe' => 'src',
    'img'    => 'src',
    'script' => 'src',
    'source' => 'src',
    'style'  => 'src',
    'video'  => 'src',
};

#========================================================================================
# global       DEFAULT_DENSITY | default image density | int
#========================================================================================
use constant DEFAULT_DENSITY => 72;

#========================================================================================
# global       INCH_IN_CM | 1 inch in cm | float
#========================================================================================
use constant INCH_IN_CM => 2.54;

#========================================================================================
# global       IMAGE_MAGICK_WARNING | regular expression indicating a message from
#                   image magick is a warning | float
#========================================================================================
use constant IMAGE_MAGICK_WARNING => qr :@ warning\/:;

#========================================================================================
# global       DEFAULT_STRINGCOMPARE_COVERAGE | threshold value that defines how similar
#               two words have to be to be rated as equal or as different. | float
#========================================================================================
use constant DEFAULT_STRINGCOMPARE_COVERAGE => 0.8;

1;
