#========================================================================================
# package      MooseObj
# state        private
#========================================================================================
package MooseObj;
use strict;
use metaclass (
    metaclass   => 'Moose::Meta::Class',
    error_class => 'DE_EPAGES::Core::API::MooseError',
);

## no critic qw(ProhibitPackageVars)

use Moose;
our $errorLine = __LINE__; has 'MyBool', 'isa' => 'Bool', 'is' => 'rw';
no Moose;

#========================================================================================
# package      main
# state        private
#========================================================================================
package main;    ## no critic (ProhibitMultiplePackages)

use Test::More;
use DE_EPAGES::Core::API::Constants qw( PERL538 );
use DE_EPAGES::Core::API::Script qw( RunScript );
use DE_EPAGES::Core::API::Error qw ( ExistsError GetError );
use strict;

sub Main {
    testOk();
    testMooseException() unless $ENV{PERL} eq '/var/epages/bin/perl';
    testRegularException();
    done_testing;
}

sub testOk {
    my $Obj = MooseObj->new();
    $Obj->MyBool(1);
    is( $Obj->MyBool(), 1, 'Obj->MyBool()' );
}

sub testMooseException {
    my $Obj = MooseObj->new();
    my ($Expected);
    eval {
        $errorLine = __LINE__ if $] < PERL538; $Obj->MyBool('Str');
        fail('should throw exception.');
    };
    if ( ExistsError() ) {
        my $Error = GetError();

        $Expected = $] < PERL538 ? 'MooseDie' : 'NativeDie';
        is( $Error->code, $Expected, 'Error->code' );

        $Expected = qr{^Attribute \(MyBool\) does not pass the type constraint because: Validation failed for 'Bool' with value "?Str"?$},
        like($Error->shortMessage, $Expected, 'Error->shortMessage');

        $Expected = $] < PERL538 ? 6 : 7;
        cmp_ok( scalar @{ $Error->callers }, '==', $Expected, 'Error->callers' );

        $Expected = qr{MooseErrorTest.t line $errorLine\b};
        like( $Error->message, $Expected, 'Error->message' );
    }
}

sub testRegularException {
    eval {
        die 'die';
        fail('should throw an exception.');    ## no critic (ProhibitUnreachableCode)
    };
    if ( ExistsError() ) {
        my $Error = GetError();
        is( $Error->code,         'NativeDie', 'Error->code: NativeDie' );
        is( $Error->shortMessage, 'die',       'Error->shortMessage: die' );
    }
}

RunScript( Sub => \&Main, Debug => 1 );
