use strict;
use DE_EPAGES::Core::API::Log qw ( GetLog LogContext SetLogLevel );
use DE_EPAGES::Core::API::Log4perl::NDCHash;
use DE_EPAGES::Core::API::Script qw ( RunScript );
use DE_EPAGES::Core::API::String qw ( sortCSVLine );

use Test::More;
use Log::Log4perl::NDC;

RunScript( Debug => 1, Sub => sub {
    my %data = (  'Test' => 'abc' );
    my $Obj = DE_EPAGES::Core::API::Log4perl::NDCHash->new(\%data);
    is($Obj . '', '{"Test":"abc"}', 'toString on NDCHash');

    my $Expected;

    LogContext({'Test1' => 'abc'}, sub {
        $Expected = '{"Test1":"abc"}';
        is(_found(), $Expected, 'LogContext Level1');
        LogContext({'Test2' => 'xyz'}, sub {
            $Expected = '{"Test1":"abc","Test2":"xyz"}';
            is(_found(), $Expected, 'LogContext Level2');
            LogContext({'Test3' => 'xx'}, sub {
                $Expected = '{"Test1":"abc","Test2":"xyz","Test3":"xx"}';
                is(_found(), $Expected, 'LogContext Level3');
            });
        });
        LogContext({'Test1' => 'def'}, sub {
            $Expected = '{"Test1":"def"}';
            is(_found(), $Expected, 'LogContext with override.');
        });
        $Expected = '{"Test1":"abc"}';
        is(_found(), $Expected, 'LogContex after override');
    });

    SetLogLevel('DEBUG', sub {
        LogContext({'Test1' => 'def'}, sub {
            GetLog('DE_EPAGES::Core::t::NDCHash')->debug('testing NDCHash');
        });
    },'DE_EPAGES::Core::t::NDCHash');

    pass('passed - check your debug.log for NDC output');
});

done_testing();

sub _found {
    my $String = Log::Log4perl::NDC->get();
    $String =~ s,^{,,;
    $String =~ s,}$,,;
    return '{'. sortCSVLine($String). '}';
}
