use strict;
use Test::Simple tests => 17;
use Perl::Critic;
use DE_EPAGES::Core::API::Script qw( RunScript );
use lib ("$ENV{EPAGES_CARTRIDGES}/DE_EPAGES/Core/Scripts");

# Tests Perl::Critic::Policy::DE_EPAGES::Security::ProhibitVariablesInSQLStatements
#

use Perl::Critic::Policy::DE_EPAGES::Security::ProhibitVariablesInSQLStatements; ##no critic(RequireCorrectImports)

sub Main {

    my $Critical;
    my $critic = Perl::Critic->new(
        -theme    => 'epages',
        -severity => 5,
        -profile  => 'NONE',
    );
    ok( defined $critic, 'Perl::Critic initialized' );

    $critic->add_policy( -policy => 'DE_EPAGES::Security::ProhibitVariablesInSQLStatements' );
    my $policyLoaded = scalar grep { $_ =~ 'DE_EPAGES::Security::ProhibitVariablesInSQLStatements' }
    $critic->policies();
    ok( $policyLoaded, 'Policy loaded' );

    # =======================================================================================
    #     Injection Tests
    # =======================================================================================
    ##no critic qw(ProhibitImplicitNewlines ProhibitMissingImports ProhibitUnusedVariables RequireInterpolationOfMetachars ProhibitLongLines)
    my $sourceCode = q(
        $dbi->do('SELECT from table' .
        " where name = $OverManLines" .
        " AND was = anderes");
    );
    $Critical = scalar $critic->critique( \$sourceCode );
    ok( $Critical, 'critical: multi-line SELECT with variable after WHERE');

    $sourceCode = q(
        my $sqlString = "inSert INTO $was";
    );
    $Critical = scalar $critic->critique( \$sourceCode );
    ok( !$Critical, 'not critical: variable as table name' );

    $sourceCode = q(
        my $sqlString = "inSert INTO $was values ($value)";
    );
    $Critical = scalar $critic->critique( \$sourceCode );
    ok( $Critical, 'critical: variable in VALUES');

    $sourceCode = q(
        my $sqlString = "inSert INTO $was SELECT * FROM othertable";
    );
    $Critical = scalar $critic->critique( \$sourceCode );
    ok( !$Critical, 'not critical: sub SELECT' );

    $sourceCode = q(
        my $kompliziert = qq(SELECT
        FROM what
        WHERE $InQQString = 10
        AND x0y
        BLA);
    );
    $Critical = scalar $critic->critique( \$sourceCode );
    ok( $Critical, 'critical: variable in qq SELECT');

    $sourceCode = q(
        die "Missing attribute alias for index $wrong" unless defined $wrong ;
    );
    $Critical = scalar $critic->critique( \$sourceCode );
    ok( !$Critical, 'not critical: variable in perl die' );

    $sourceCode = q(
        print "This is not an mysql update, $wrong dude!";
    );
    $Critical = scalar $critic->critique( \$sourceCode );
    ok( !$Critical, 'not critical: variable in perl print' );

    $sourceCode = q(
        $dbi->do('INSERT INTO mailtypetemplate (mailtypetemplateid, pagetypeid, hastofield, hassubject, hasadditionaltext, isalwaysactive, hasheader) VALUES (?, ?, ?, ?, ?, ?, ?)',
             [ @{$wrong}{qw( MailTypeTemplateID PageTypeID HasToField HasSubject HasAdditionalText IsAlwaysActive HasHeader )} ]
    );
    );
    $Critical = scalar $critic->critique( \$sourceCode );
    ok( !$Critical, 'not critical: SQL only with ?' );

    $sourceCode = q{ $dbi->do("alter table $table DROP $column") };
    $Critical = scalar $critic->critique( \$sourceCode );
    ok( !$Critical, 'not critical: alter table without WHERE' );

    $sourceCode = <<'END_CODE';
TestInteger( 'ShopID', $ShopID );
$dbi->createTempTable('export_ids (objectid int)');
$dbi->do("insert into $exportids (objectid) select objectid from object where siteid = ?", [$ShopID]);
END_CODE
    $Critical = scalar $critic->critique( \$sourceCode );
    ok( !$Critical, 'not critical: SQL with ? replaced by variables' );

    $sourceCode = q (
        $dbi->do("delete from $tableids where objectid in (select d.objectid from $deleteids d)");
    );
    $Critical = scalar $critic->critique( \$sourceCode );
    ok( !$Critical, 'not critical: sub SELECT variables without WHERE' );

    $sourceCode = q (
        $dbi->do("delete from $tableids where objectid in (select d.objectid from $deleteids d where a = $var)");
    );
    $Critical = scalar $critic->critique( \$sourceCode );
    ok( $Critical, 'critical: sub SELECT with variables after WHERE' );

    $sourceCode = q (
        $dbi->do("delete from $tableids where objectid in (select d.objectid from $deleteids d where a = b) and a = $b");
    );
    $Critical = scalar $critic->critique( \$sourceCode );
    ok( $Critical, 'critical: sub SELECT with variables after SELECT' );

    $sourceCode = q (
        $dbi->do("delete from $tableids where objectid = " . $b);
    );
    $Critical = scalar $critic->critique( \$sourceCode );
    ok( $Critical, 'critical: concatenated SQL with variable' );

    $sourceCode = q (
        $dbi->do("delete from $tableids2 where objectid = " . $b);
    );
    $Critical = scalar $critic->critique( \$sourceCode );
    ok( $Critical, 'critical: concatenated SQL with variable (2)' );

}

RunScript( Sub => \&Main );
