﻿#!perl -w
use strict;
use Test::More tests => 53;
use DE_EPAGES::Core::API::Unicode qw( Unicode2Alias Unicode2Filename Unicode2Path Unicode2DirectoryName );


use utf8;

# Unicode2Alias
is( Unicode2Alias('ÀÁÂÃÄÅ'), 'AAAAAeA', 'Unicode2Alias(Example)' ); #example of docu
is( Unicode2Alias('Viele Köche'), 'Viele_Koeche', 'Unicode2Alias(umlauts)');#example of docu
is( Unicode2Alias('Радуга позволяет создавать'), 'Raduga_pozvolyait_sozdavat', 'Unicode2Alias(Raduga)');
is( Unicode2Alias('Branschrådet Svensk  massörers'), 'Branschradet_Svensk_massoerers', 'Unicode2Alias(Swedish)');
is( Unicode2Alias('0123456789!@#$%^&*()'), '0123456789', 'Unicode2Alias(special chars)' );
is( Unicode2Alias('окно'), 'okno', 'Unicode2Alias = okno' );
# some Latin 7
is( Unicode2Alias('ÀÁÂÃÄÅÆÇÈÉÊËÌÍÎÏÐÑÒÓÔÕÖ×ØÙÚÛÜÝÞßàáâãäåæçèéêëìíîïðñòóôõö÷øùúûüýþÿ'), 'AAAAAeAeCEEEEIIIIDNOOOOOexOUUUUeYPssaaaaaeaeceeeeiiiidnoooooeouuuueypy', 'Unicode2Alias(Latin7)');
# some Cyrillic 8
is( Unicode2Alias('ЁЂЃЄЅІЇЈЉЊЋЌЎЏАБВГДЕЖЗИЙКЛМНОПРСТУФХЦЧШЩЪЫЬЭЮЯабвгдежзийклмнопрстуфхцчшщъыьэюяёђѓєѕіїјљњћќўџ'), 'IoDjGjIDzIYiJLjNjTshKjUDzhABVGDIZhZIIKLMNOPRSTUFHaTsChShaShchaYruYuYaabvgdizhziiklmnoprstufhatschshashchayruyuyaiodjgjidziyijljnjtshkjudzh', 'Unicode2Alias(Cyrillic8)');
# some Greek 9 http://en.wikipedia.org/wiki/Greek_gods
is( Unicode2Alias('Aφροδίτη'), 'Aphrodite', 'Unicode2Alias = Aphrodite');
is( Unicode2Alias('Aπόλλων'), 'Apollon', 'Unicode2Alias = Apollon');
is( Unicode2Alias('Άρης'), 'Ares', 'Unicode2Alias = Ares');
is( Unicode2Alias('Άρτεμις'), 'Artemis', 'Unicode2Alias = Artemis');
is( Unicode2Alias('Αθηνά'), 'Athena', 'Unicode2Alias = Athena');
is( Unicode2Alias('Δήμητρα'), 'Demetra', 'Unicode2Alias = Demetra');
is( Unicode2Alias('Άδης'), 'Ades', 'Unicode2Alias = Ades'); # Hades
is( Unicode2Alias('Ήφαιστος'), 'Hephaistos', 'Unicode2Alias = Hephaistos');
is( Unicode2Alias('Ήρα'), 'Hera', 'Unicode2Alias = Hera');
is( Unicode2Alias('Ερμής'), 'Hermes', 'Unicode2Alias = Hermes');
is( Unicode2Alias('Εστία'), 'Hestia', 'Unicode2Alias = Hestia');
is( Unicode2Alias('Ποσειδων'), 'Poseidon', 'Unicode2Alias = Poseidon');
is( Unicode2Alias('Ζεύς'), 'Zeus', 'Unicode2Alias = Zeus');

cmp_ok(Unicode2Alias(' '), 'eq', '_', 'SPACE');
cmp_ok(Unicode2Alias(chr(0)), 'eq', '_0_', 'NULL');
cmp_ok(Unicode2Alias(chr(8)), 'eq', '_8_', 'BACKSPACE');
cmp_ok(Unicode2Alias(chr(9)), 'eq', '_', 'HORIZONTAL TAB');
cmp_ok(Unicode2Alias(chr(10)), 'eq', '_', 'LINE FEED');
# The vertical tab is part of \s in Perl 5.18
my $Expected = $] < 5.018 ? '_11_' : '_';
cmp_ok(Unicode2Alias(chr(11)), 'eq', $Expected, 'VERTICAL TAB');
cmp_ok(Unicode2Alias(chr(12)), 'eq', '_', 'FORM FEED');
cmp_ok(Unicode2Alias(chr(13)), 'eq', '_', 'CARRIAGE RETURN');
cmp_ok(Unicode2Alias(chr(14)), 'eq', '_14_', 'SHIFT OUT');
cmp_ok(Unicode2Alias(chr(0xfff0)), 'eq', '_65520_', 'UNDEFINED');

# Unicode2Filename:
cmp_ok(Unicode2Filename('/asd/!#Hündchenölgemälde.jpg'), 'eq', 'asdHuendchenoelgemaelde.jpg', 'replace umlauts for filename');
cmp_ok(Unicode2Filename('/a_s_d/!#Hündchen Ölgemälde-ÉÊËÌÍ#{}(b+).создавать'), 'eq', 'a_s_dHuendchen_Oelgemaelde-EEEII_123__125_b.sozdavat', 'replace umlauts for filename');
cmp_ok(Unicode2Filename('/a_s_d/!#Hündchen Ölgemälde-ÉÊËÌÍ#{}(b+).создавать', '!'), 'eq', 'a_s_d!Huendchen_Oelgemaelde-EEEII_123__125_b.sozdavat', 'replace umlauts for filename');
cmp_ok(Unicode2Filename('{'), 'eq', '_123_', 'single bracket');
cmp_ok(Unicode2Filename('Ερμής.Ζεύς'), 'eq', 'Hermes.Zeus', 'greek Hermes.Zeus');
cmp_ok(Unicode2Filename('/:*?"<>|&\'%#/z/z/z/'.chr(0x0019)), 'eq', '_124_zzz_25_', 'special chars, 124 is pipe');
cmp_ok(Unicode2Filename('/asd,ded/asdasd.,de'), 'eq', 'asddedasdasd.de', 'kommas');

# Unicode2DirectoryName:
cmp_ok(Unicode2DirectoryName('/asd/!#Hündchenölgemälde.jpg'), 'eq', 'asdHuendchenoelgemaeldejpg', 'replace umlauts for filename');
cmp_ok(Unicode2DirectoryName('/a_s_d/!#Hündchen Ölgemälde-ÉÊËÌÍ#{}(b+).создавать'), 'eq', 'a_s_dHuendchen_Oelgemaelde-EEEII_123__125_bsozdavat', 'replace umlauts for filename');
cmp_ok(Unicode2DirectoryName('/a_s_d/!#Hündchen Ölgemälde-ÉÊËÌÍ#{}(b+).создавать', '!'), 'eq', 'a_s_d!Huendchen_Oelgemaelde-EEEII_123__125_bsozdavat', 'replace umlauts for filename');
cmp_ok(Unicode2DirectoryName('{'), 'eq', '_123_', 'single bracket');
cmp_ok(Unicode2DirectoryName('Ερμής.Ζεύς'), 'eq', 'HermesZeus', 'greek Hermes.Zeus');
cmp_ok(Unicode2DirectoryName('/:*?"<>|&\'%#/z/z/z/'.chr(0x0019)), 'eq', '_124_zzz_25_', 'special chars, 124 is pipe');
cmp_ok(Unicode2DirectoryName('d/r/eiAUOA/Sd1zz\zz23'), 'eq', 'dreiAUOASd1zz_92_zz23', 'many slashes');
cmp_ok(Unicode2DirectoryName('/asd,ded/asdasd.,de'), 'eq', 'asddedasdasdde', 'kommas');

# Unicode2Path:
cmp_ok(Unicode2Path('/asd/!#Hündchenölgemälde.jpg'), 'eq', '/asd/Huendchenoelgemaeldejpg', 'replace umlauts for directory');
cmp_ok(Unicode2Path('/a_s_d/!#Hündchen Ölgemälde-ÉÊËÌÍ#{}(b+).создавать'), 'eq', '/a_s_d/Huendchen_Oelgemaelde-EEEII_123__125_bsozdavat', 'replace umlauts for directory');
cmp_ok(Unicode2Path('/a_s_d/!#Hündchen Ölgemälde-ÉÊËÌÍ#{}(b+).создавать', '!.'), 'eq', '/a_s_d/!Huendchen_Oelgemaelde-EEEII_123__125_b.sozdavat', 'replace umlauts for dir with allow chars');
cmp_ok(Unicode2Path('/{/'), 'eq', '/_123_', 'truncated last slash');
cmp_ok(Unicode2Path('\t\Ερμής/Ζεύς.Νύξ'), 'eq', '/t/Hermes/ZeusNux', 'greek \t\Hermes/Zeus.Nux');
cmp_ok(Unicode2Path(':*?"<>|&\'%#/z/z/z/'.chr(0x0019)), 'eq', '_124_/z/z/z/_25_', 'special chars');
cmp_ok(Unicode2Path('/asd,ded/asdasd.,de'), 'eq', '/asdded/asdasdde', 'kommas');

