package DE_EPAGES::Core::t::unit::Log; ##no critic qw(RequireCommentForExportedFunctions)
use base DE_EPAGES::Core::API::Test;
use DE_EPAGES::Core::API::Error qw ( Error ExistsError GetError );
use DE_EPAGES::Core::API::Log qw ( LogContext LogError );
use DE_EPAGES::Core::API::String qw ( sortCSVLine );

use Test::Exception;
use Test::More;

use strict;

sub test {
    my $self = shift;
    $self->test_nestedContext_withError ();
    $self->test_nestedContext_withError_ToLogFile ();
    $self->test_noContext_withError_ToLogFile ();
    done_testing();
}

sub test_nestedContext_withError {
    my $self = shift;

    # vars
    my $Context_begin;
    my $Context_end;
    my $Context_appSrv_begin;
    my $Context_appSrv_end;
    my $Context_viewAction_begin;
    my $Context_viewAction_end;
    my $Context_Implementation_begin;
    my $hContext_Implementation_errorLog;

    #call
    $Context_begin = _NDCget();
    LogContext({'AppServerLevelKey' => 'AppServerLevelValue'}, sub{
      $Context_appSrv_begin = _NDCget();

      LogContext({'ViewActionLevelKey' => 'ViewActionLevelValue'}, sub{
          $Context_viewAction_begin = _NDCget();
          eval {

              LogContext({'ImplementationLevelKey' => 'ImplementationLevel'}, sub{
                $Context_Implementation_begin = _NDCget();
                Error('uups', {'Message' => 'ImplementationLevel Error'});
              });# Implementation level

          };
          if (ExistsError()) {
              my $Error = GetError();
              $hContext_Implementation_errorLog = $Error->getNdcHash();
          }

          $Context_viewAction_end = _NDCget();

      }); # ViewAction level
      $Context_appSrv_end = _NDCget();

    }); # AppServer level
    $Context_end = _NDCget();

    # test
    my $Expected = '[undef]';
    is($Context_begin, $Expected, 'NDC is empty when init');
    $Expected = '{"AppServerLevelKey":"AppServerLevelValue"}';
    is($Context_appSrv_begin, $Expected, 'NDC contains context for appsrv level');
    $Expected = '{"AppServerLevelKey":"AppServerLevelValue","ViewActionLevelKey":"ViewActionLevelValue"}';
    is( $Context_viewAction_begin, $Expected, 'NDC contains context for appsrv/viewaction level');
    $Expected = '{"AppServerLevelKey":"AppServerLevelValue","ImplementationLevelKey":"ImplementationLevel","ViewActionLevelKey":"ViewActionLevelValue"}';
    is( $Context_Implementation_begin, $Expected, 'NDC contains context for appsrv/viewaction/implementation level');
    $Expected = { 'AppServerLevelKey' => 'AppServerLevelValue',
                  'ImplementationLevelKey' => 'ImplementationLevel',
                  'ViewActionLevelKey' => 'ViewActionLevelValue' };
    is_deeply($hContext_Implementation_errorLog, $Expected, 'NDC error context is complete');
    $Expected = '{"AppServerLevelKey":"AppServerLevelValue","ViewActionLevelKey":"ViewActionLevelValue"}';
    is( $Context_viewAction_end, $Expected, 'NDC contains context only for appsrv/viewaction level');
    $Expected = '{"AppServerLevelKey":"AppServerLevelValue"}';
    is($Context_appSrv_end, $Expected, 'NDC contains context only for appsrv level');
    $Expected = '[undef]';
    is($Context_end, $Expected,'NDC is empty after context');
}

sub test_nestedContext_withError_ToLogFile {
    my $self = shift;
    lives_ok( sub {
        warn 'before Context';
        LogContext({'AppServerLevelKey' => 'AppServerLevelValue'}, sub{
            warn 'begin AppSrv level';
            LogContext({'ViewActionLevelKey' => 'ViewActionLevelValue'}, sub{
                warn 'begin ViewAction level';
                eval {
                    LogContext({'ImplementationLevelKey' => 'ImplementationLevel'}, sub{
                        warn 'begin Implementation level';
                        Error('ImplementationLevel Error');
                    });# Implementation level
                };
                LogError(GetError()) if ExistsError();
                warn 'end ViewAction level';
            }); # ViewAction level
            warn 'end AppSrv level';
        }); # AppServer level
        warn 'after Context';
    }, 'ImplementationLevel Error (see info.log)' );
}

sub test_noContext_withError_ToLogFile {
    my $self = shift;

    lives_ok( sub {
        eval { Error('No Context Error'); };
        LogError(GetError()) if ExistsError();
    }, 'No Context Error (see info.log)' );
}

sub _NDCget {
    my $String = Log::Log4perl::NDC->get();
    my ($Open, $Close);
    $String =~ s,^{,, and $Open = '{';
    $String =~ s,}$,, and $Close = '}';
    $String = sortCSVLine($String);
    $String = "$Open$String$Close";
    return $String;
}

__PACKAGE__->runScript();
