#==============================================================================
# package      DE_EPAGES::MolliePayments::Hooks::Attribute
# state        private
#==============================================================================
package DE_EPAGES::MolliePayments::Hooks::Attribute;

use DE_EPAGES::Object::API::Factory qw (LoadObjectByPath GetRootObjectID);
use DE_EPAGES::ThirdPartyConfig::API::Constants qw (ATTRIBUTES_CATEGORY_PAYMENT);

use strict;



#========================================================================================
# function     OnGetThirdPartyConfigAttributes
# state        public
#----------------------------------------------------------------------------------------
# syntax       OnGetThirdPartyConfigAttributes($hParams);
#----------------------------------------------------------------------------------------
# description  API_GetThirdPartyConfigAttributes triggered, register attributes to be
#               configured in PBO
#----------------------------------------------------------------------------------------
# input        $hParams | hook arguments, used keys are
#               <ul>
#                   <li>AttributesCategory
#                   <li>LanguageID
#               </ul> | ref.hash.*
#========================================================================================
sub OnGetThirdPartyConfigAttributes {
    my ($hParams) = @_;

    return unless $hParams->{'AttributesCategory'} eq ATTRIBUTES_CATEGORY_PAYMENT;

    my $PaymentType  = LoadObjectByPath('/PaymentTypes/MolliePayments');
    my $RootObjectID = GetRootObjectID();

    push(@{$hParams->{'Attributes'}}, {
        'SectionID'   => $PaymentType->alias(),
        'SectionName' => $PaymentType->get('NameOrAlias', $hParams->{'LanguageID'}),
        'Attributes' => [
            {'ObjectID' => $PaymentType->id(), 'AttributeAlias' => 'Logging'},
            {'ObjectID' => $RootObjectID, 'AttributeAlias' => 'MolliePayments_RegisterURL'},
            {'ObjectID' => $RootObjectID, 'AttributeAlias' => 'MolliePayments_AppRedirectURL'},
            {'ObjectID' => $RootObjectID, 'AttributeAlias' => 'MolliePayments_AppClientID'},
            {'ObjectID' => $RootObjectID, 'AttributeAlias' => 'MolliePayments_AppClientSecret'},
            {'ObjectID' => $RootObjectID, 'AttributeAlias' => 'MolliePayments_OrganisationToken_ClientLinks'},
        ]
    });

    return;
}

1;