#LOCAL("PPCPParameter", #ExtractedPayPalPPCPParameters)
#IF(#PPCPParameter)
(function () {
  var MinMaxAllowed,
      InCompatibleShippingMethods,
      ShippingMethodLandRestrictions,
      buttonContainer = 'div#paypal-button-container';

      try {
        MinMaxAllowed = JSON.parse('#PPCPParameter.MinMaxAllCurrencies[0]');
      } catch (ex) {
      }
      try {
        InCompatibleShippingMethods = JSON.parse('#PPCPParameter.InCompatibleShippingMethods[0]');
        ShippingMethodLandRestrictions = JSON.parse('#PPCPParameter.ShippingMethodLandRestrictions[0]');
      } catch (ex) {
      }

  var initPayPal = function() {
    var isOk = 1;

    if (window.paypal) {
      try {
        var FUNDING_SOURCES = [
          #IF(#PPCPParameter.EnableCartButtons)
            paypal.FUNDING.PAYPAL,
            paypal.FUNDING.PAYLATER
          #ENDIF
        ];

        FUNDING_SOURCES.forEach(function(fundingSource) {
            var button = paypal.Buttons({
              style: {
                layout: 'vertical',
                color: fundingSource === paypal.FUNDING.PAYPAL || fundingSource === paypal.FUNDING.PAYLATER ? 'gold' : 'white',
                shape: 'rect',
                label: 'checkout',
                height: 55,
              },
              fundingSource: fundingSource,
              createOrder: function (data, actions) {
                return xhrRequest();
              },
              onApprove: function (data, actions) {
                xhrRequestApprove(fundingSource);
              },
              onCancel: function (data, actions) {
                console.log("Customer cancelled the PayPal Checkout Flow");
              },
              onError: function () {
                console.log("An Error occurred as part of the PayPal JS SDK");
              },
              onShippingChange: function(data, actions) {
                if ( data.selected_shipping_option.id && ShippingMethodLandRestrictions[data.selected_shipping_option.id] ) {
                  if ( ShippingMethodLandRestrictions[data.selected_shipping_option.id][data.shipping_address.country_code] ) {
                    return actions.resolve();
                  } else {
                    return actions.reject();
                  }
                }
                return actions.resolve();
              }
            });

            if ( button.isEligible() ) {
                button.render(buttonContainer);
            }
        });

        /* #REM
            Pay Later Banner
        #ENDREM */
        #IF(#PPCPParameter.EnablePayLaterBanners)
        if ( cart ) {
            var elem = document.querySelector(buttonContainer);
            if ( elem ) {
              elem.setAttribute('data-pp-amount', cart.grandTotal.amount);
            }
            paypal.Messages({
              style: {
                layout: 'text',
                logo: {
                  type: 'primary'
                },
                text: {
                  align: 'center'
                }
              },
            }).render(buttonContainer);
        }
        #ENDIF

      } catch (e) {
        isOk = 0;
        console.log(e);
      }
    } else {
      isOk = 0;
    }

    if (!isOk) {
      #REM
      //document.getElementById('#'+'PayPalPPCPErrorLink').style.display = 'block';
      //console.log('IS_NOT OK', document.getElementById('#'+'PayPalPPCPErrorLink'));
      #ENDREM
    }
  },

  xhrRequest = function() {
    return new Promise(function(resolve, reject) {
      if (XMLHttpRequest && cart) {
        var xhr = new XMLHttpRequest(),
          targetUrl = '#PPCPParameter.CreateOrderUrl[0]',
          data = {
            'BasketGUID': cart.cartId,
          };

        xhr.open('POST', targetUrl, true);
        xhr.setRequestHeader('Content-Type', 'application/json');

        xhr.addEventListener('load', function () {
          if (xhr.status < 200 || xhr.status >= 300) {
            reject(new Error(xhr.responseText + xhr.statusText));
          } else {
            try {
              var response = JSON.parse(xhr.responseText);

              if (response.success) {
                resolve(response.orderID);
              } else {
                reject(new Error(xhr.responseText + xhr.statusText));
              }
            } catch (e) {
              reject(e);
            }
          }
        });

        xhr.send(JSON.stringify(data));
      }
    })
  },

  xhrRequestApprove = function(fundingSource) {
    if (XMLHttpRequest && cart) {
      var xhr = new XMLHttpRequest(),
        targetUrl = '#PPCPParameter.ApproveOrderUrl[0]',
        data = {
          'PickupToken': '#PPCPParameter.ApprovePPCPToken',
          'BasketGUID': cart.cartId,
          'LocaleID': shopLocale,
          'FundingSource': fundingSource,
        };

      xhr.open('POST', targetUrl, true);
      xhr.setRequestHeader('Content-Type', 'application/json');

      xhr.addEventListener('load', function () {
        if (xhr.status < 200 || xhr.status >= 300) {
          console.warn(new Error(xhr.responseText + xhr.statusText));
        } else {
          try {
            var response = JSON.parse(xhr.responseText);

            if (response.success) {
              window.location = response.redirect;
            } else {
              console.warn(new Error(xhr.responseText + xhr.statusText));
            }
          } catch (e) {
            console.log(e);
          }
        }
      });

      xhr.send(JSON.stringify(data));
    }
  };

  if (window.eComEventTarget) {
    var cart,
        script,
        shopLocale,
        ppcpUpdateCart = function (e) {
          var container = document.querySelector(buttonContainer),
              timeout = 500,
              CompatibleWithCurrentShipping = 1;

          if ( container ) {
            timeout = 0;
          }

          /* #REM
            The first time this triggers, the selector returns nothing, presumably because React is still building the page
          #ENDREM */
          setTimeout(function(){

            if ( ! container ) {
                container = document.querySelector(buttonContainer);
            }
            var cartHasItems = typeof cart.productLineItems === 'object' && cart.productLineItems.length > 0 ? true : false;

            if ( container && cartHasItems ) {
              /* #REM
                Hide/Show the button div depending on whether the basket satisfies the Min/Max values of the payment method
              #ENDREM */
              var cartCurrencyId = cart.grandTotal.currency;

              container.setAttribute('data-pp-amount', cart.grandTotal.amount);

              /* #REM
                If the cart's shipping method is on the list of incompatible shipping methods with our PPCP method
              #ENDREM */
              if ( cart.shippingLineItem.shippingMethod.name ) {
                  try {
                    var shippingNameEncoded = btoa(cart.shippingLineItem.shippingMethod.name);
                    if ( InCompatibleShippingMethods[shippingNameEncoded] ) {
                      CompatibleWithCurrentShipping = 0;
                    }
                  } catch (ex) {
                  }
              }

              if ( ! CompatibleWithCurrentShipping ) {
                container.style.display = 'none';
              } else if ( typeof MinMaxAllowed === 'object' && MinMaxAllowed[cartCurrencyId]['MinPriceValue'] && cart.grandTotal.amount < MinMaxAllowed[cartCurrencyId]['MinPriceValue'] ) {
                container.style.display = 'none';
              } else if ( typeof MinMaxAllowed === 'object' && MinMaxAllowed[cartCurrencyId]['MaxPriceValue'] && cart.grandTotal.amount > MinMaxAllowed[cartCurrencyId]['MaxPriceValue'] ) {
                container.style.display = 'none';
              } else {
                container.style.display = '';
              }

              if (window.paypal && e.type === 'cart:view') {
                initPayPal();
              }
            }

          }, timeout);
        };

    /* #REM
        Product page bannerz
    #ENDREM */
    #IF(#PPCPParameter.EnablePayLaterBanners)
      document.head.insertAdjacentHTML('beforeend', '<style>div.product-button-container span[id*="zoid-paypal-message"]{margin-top:20px;width:100%;}</style>');
      window.eComEventTarget.addEventListener('product:view', function(event) {
        var product = event.detail.product,
            productSelector = 'div.product-button-container',
            container = document.querySelector(productSelector),
            timeout = 500;

        if ( container ) {
          timeout = 0;
        }

        /* #REM
            The first time this triggers, the selector returns nothing, presumably because React is still building the page
        #ENDREM */
        setTimeout(function(){
            if ( ! container ) {
              container = document.querySelector(productSelector);
            }

            if ( container ) {
              container.setAttribute('data-pp-amount', product.price.amount);

              paypal.Messages({
                  style: {
                    layout: 'text',
                    logo: {
                      type: 'primary'
                    }
                  },
              }).render(productSelector);
            }

        }, timeout);
      });
    #ENDIF

    window.eComEventTarget.addEventListener('cart:view', function(event) {
      cart = event.detail.cart;
      shopLocale = event.detail.shopLocale;

      ppcpUpdateCart(event);
    });

    window.eComEventTarget.addEventListener('cart:update', function(event) {
      cart = event.detail.cart;

      ppcpUpdateCart(event);
    });

    script = document.createElement('script');
    script.src = 'https://www.paypal.com/sdk/js?#IF(#PayPalPPCPIntegrationDate)integration-date=#PayPalPPCPIntegrationDate&#ENDIF#SPACE[0]client-id=#PPCPParameter.ClientID&currency=#PPCPParameter.CurrencyID&intent=capture&components=buttons,funding-eligibility#IF(#PPCPParameter.EnablePayLaterBanners),messages#ENDIF#IF(#PPCPParameter.EnablePayLater)&enable-funding=paylater#ENDIF';
    script.setAttribute("defer", true);
    script.setAttribute('data-partner-attribution-id', '#PPCPParameter.PartnerBNCode');
    document.body.appendChild(script);

    /* #REM
        pay later message span tags are displayed inline, causing it to be too wide, also affecting the buttons above it, so make it a block element instead
    #ENDREM */
    document.head.insertAdjacentHTML('beforeend', '<style>div#paypal-button-container span[id*="zoid-paypal-message"]{display: block;}</style>');
  }
})();
#ENDIF
#ENDLOCAL #REM /* LOCAL("PPCPParameter", ExtractedPayPalPPCPParameters) */ #ENDREM
