use Test::Simple tests => 4;
use strict;
use DE_EPAGES::Core::API::Error qw ( GetError ExistsError Error );
use DE_EPAGES::Core::API::Script qw ( RunScript );
use DE_EPAGES::Core::API::File qw ( normalizeXML );
use DE_EPAGES::Database::API::Connection qw ( ConnectTestSite DisconnectSite );
use DE_EPAGES::Trigger::API::Trigger qw ( TriggerHook );
use DE_EPAGES::Trigger::API::XML::HookImportHandler qw ( ImportHooks );
use DE_EPAGES::Trigger::API::XML::HookDeleteHandler qw ( DeleteHooks );
use DE_EPAGES::Trigger::API::Table::Hook qw ( InsertHook GetPKeyHooksByName DeleteHook );

our $CALLED = 0;

sub Main {
    local $| = 1;
    local $ENV{SCRIPT_NAME} = __FILE__;
    ConnectTestSite();
    Cleanup();

    Test_Import();
    Test_Trigger();
    Test_Export();
    Test_ExportFalse();

    Cleanup();
    DisconnectSite();
}

sub Test_Import {
    eval {
        ImportHooks('01_s.xml');
        ImportHooks('01_s.xml');
    };
    ok( ExistsError() ? 0 : 1, 'Test_Import()');
}

sub Test_Trigger {
    eval {
        TriggerHook('TestHook', { 'TestParameter'=>'Param' });
        Error('NO_HOOK_NOTCALLED') unless $CALLED;
        $CALLED = 0;
        TriggerHook('TestHookPackage', { 'TestParameter'=>'Param' });
        Error('NO_HOOK_NOTCALLED') unless $CALLED;
    };
    ok( ExistsError() ? 0 : 1, 'Test_Trigger()');
}

sub Test_Export {
    ImportHooks('01_s.xml');
    my $Driver = TestDriver->new();
    $Driver->exportXML('01_e.xml',{ encoding => 'iso-8859-1' });

    my $Expected = normalizeXML('01_s.xml');
    my $Found = normalizeXML('01_e.xml');
    ok($Found eq $Expected, 'Test_Export()');
}

sub Test_ExportFalse {
    ImportHooks('01_s.xml');
    InsertHook({'Name'=>'TestHook3'});
    my $Driver = TestDriver->new();
    $Driver->exportXML('01_f.xml',{ encoding => 'iso-8859-1' });

    my $Expected = normalizeXML('01_s.xml');
    my $Found = normalizeXML('01_f.xml');
    ok($Found ne $Expected, 'Test_Export()');
}

sub TestHookProc {
    my ($args) = @_;
    Error('NO_HOOK_NAME') unless defined $args->{'HookName'}      and $args->{'HookName'} eq 'TestHook';
    Error('NO_HOOK_PARA') unless defined $args->{'TestParameter'} and $args->{'TestParameter'} eq 'Param';
    $CALLED = 1;
}

sub Cleanup {
    DeleteHooks('01_s.xml');
    my $IDs = GetPKeyHooksByName('TestHook3');
    DeleteHook($_) foreach @$IDs;
    unlink '01_e.xml';
    unlink '01_f.xml';
}

RunScript(
    'Sub' => \&Main
);

##no critic (RequireCommentForExportedFunctions)
package HookT;
##use critic
use strict;
use DE_EPAGES::Core::API::Error qw ( Error );##no critic (RequireCorrectImports)

sub TestHookProc {
    my ($args) = @_;
    Error('NO_HOOK_NAME') unless defined $args->{'HookName'}      and $args->{'HookName'} eq 'TestHookPackage';
    Error('NO_HOOK_PARA') unless defined $args->{'TestParameter'} and $args->{'TestParameter'} eq 'Param';
    $CALLED = 1;
}

##no critic(ProhibitMultiplePackages)
package TestDriver;
##use critic
use base qw( DE_EPAGES::Trigger::API::XML::HookExportDriver );

use strict;
use DE_EPAGES::Trigger::API::Table::Hook qw( GetPKeyHooksByName );##no critic (RequireCorrectImports)

sub AllHooks {
    my $self = shift;
    my $IDs = GetPKeyHooksByName('TestHook');
    $self->addElement( 'Hook', { 'HookID' => $_ } ) foreach @$IDs;
    $IDs = GetPKeyHooksByName('TestHookPackage');
    $self->addElement( 'Hook', { 'HookID' => $_ } ) foreach @$IDs;
    $IDs = GetPKeyHooksByName('TestHook3');
    $self->addElement( 'Hook', { 'HookID' => $_ } ) foreach @$IDs;
}

# get attributes for the document tag
sub epages {
    return {};
}

sub addHookHooks {
    my $self = shift;
    $self->addHook( 'Hook' => 'HookFunctionsByHook' );
}

sub parse {
    my $self = shift;

    $self->addHook( 'epages' => 'AllHooks' );
    $self->addHookHooks;

    $self->SUPER::start_document();
    $self->xml_decl({'Version'=>'1.0', 'Encoding'=>$self->encodeTo});
    $self->characters({'Data' => "\n"});
    $self->addElement( 'epages' );
    $self->SUPER::end_document();
}

1;
