#==============================================================================
# package      DE_EPAGES::XML::API::Converter
# state        public
#------------------------------------------------------------------------------
# description  This module is used by DE_EPAGES::XML::API::SAXWriter
#==============================================================================
package DE_EPAGES::XML::API::Converter;

use strict;
use Encode qw (resolve_alias encode);
use DE_EPAGES::Core::API::Constants qw (PERL538);
use DE_EPAGES::Core::API::Error qw (Error);

sub ErrorDefinition {
    return {
        'EncodeFrom' => 'Encoding is utf8 (not #Encoding), standard perl strings.',
        'EncodeUnknown' => 'Encoding is unknown (#Encoding).',
    }
}

sub new {
    my $self = shift;
    my ($EncodeFrom, $EncodeTo) = @_;
    Error('EncodeFrom',    { 'Encoding'=> $EncodeFrom}) unless $EncodeFrom eq 'utf-8';
    Error('EncodeUnknown', { 'Encoding'=> $EncodeTo  }) unless resolve_alias($EncodeTo);
    return bless {
        'EncodeFrom' => $EncodeFrom,
        'EncodeTo'   => $EncodeTo,
    }, $self;
}

sub convert {
    my $self = shift;
    my ($data) = @_;
    $data = encode($self->{'EncodeTo'}, $data, Encode::FB_XMLCREF) if $] < PERL538;
    return $data;
}


1;
