package DE_EPAGES::XML::t::BaseExportDriver; ##no critic (RequireCommentForExportedFunctions)

use base DE_EPAGES::Core::API::Test;

use strict;
use Test::More tests => 2;
use XML::SAX::Writer;
use XML::SemanticDiff;
use XML::Filter::DataIndenter;
use DE_EPAGES::Core::API::String qw ( sortXMLAttributes );
use DE_EPAGES::XML::API::BaseExportDriver;

sub TestNamespace {

    my $Driver = My::TestNamespace->new;
    my $Output = '';
    my $Writer = XML::SAX::Writer->new({
        Writer         => 'DE_EPAGES::XML::API::SAXWriter',
        Output         => \$Output,
        EncodeTo       => 'utf-8',
    });
    my $Filter = XML::Filter::DataIndenter->new( 'Handler' => $Writer );
    $Driver->set_handler( $Filter );
    $Driver->parse();
    $Output =~ s/"/'/g;
    $Output =~ s/\n\n/\n/gs;

    my $Expected = <<'END_XML';
<?xml version='1.0' encoding='utf-8'?>
<s1:foo id='1' xmlns:s1='http://acme.com/serviceuri/1' xmlns:s2='http://acme.com/serviceuri/2'>
  <s1:bar>text</s1:bar>
  <s1:baz>
    <s1:poo>more_text</s1:poo>
  </s1:baz>
  <s2:foo id='2'>text</s2:foo>
  <foo id='3'>text</foo>
</s1:foo>
END_XML

    is(_sortAttributesInXML($Output), $Expected, 'Export with namespace on elements' );
}

sub TestExport {

    my $XML = '';
    my $Writer = XML::SAX::Writer->new({
        Writer => 'DE_EPAGES::XML::API::SAXWriter',
        Output => \$XML,
    });
    my $Filter = XML::Filter::DataIndenter->new( 'Handler' => $Writer );
    my $Driver = DE_EPAGES::XML::API::BaseExportDriver->new( Handler => $Filter );

    $Driver->writeDocument( sub {
        $Driver->writeElement('Company', { Name => 'ePages' }, sub {
            $Driver->writeElement('Office', { Country => 'Germany', City => 'Jena' }, 'Development Crew');
            $Driver->writeElement('Office', { Country => 'Germany', City => 'Hamburg' }, 'Water Head');
        });
    });

    my $Expected = <<'END_XML';
<?xml version='1.0' encoding='utf-8'?>
<Company Name='ePages'>
  <Office Country='Germany' City='Jena'>Development Crew</Office>
  <Office Country='Germany' City='Hamburg'>Water Head</Office>
</Company>
END_XML

    my $diff = XML::SemanticDiff->new(keeplinenums => 1);
    my @Changes = ($diff->compare($XML, $Expected));
    ok( 0 == scalar @Changes, 'No changes' );
    foreach my $change ( @Changes ) {
        print "# $change->{message} (between lines $change->{startline} and $change->{endline})\n";
    }
}

sub test {
    TestNamespace();
    TestExport();
}

sub _sortAttributesInXML {
    my ($XML) = @_;
    my $SortedXML;
    foreach my $XMLLine (split(/\n/, $XML)) {
        $SortedXML .= sortXMLAttributes($XMLLine). "\n";
    }
    return $SortedXML;
}

__PACKAGE__->runScript;

package My::TestNamespace; ## no critic qw( ProhibitMultiplePackages )
use base 'DE_EPAGES::XML::API::BaseExportDriver';
use strict;

sub drive {
    my $self = shift;

    $self->prefixMapping( {
        s1 => 'http://acme.com/serviceuri/1',
        s2 => 'http://acme.com/serviceuri/2',
    }, sub {

        $self->namespace('http://acme.com/serviceuri/1', sub {
            $self->writeElement( 'foo', { id => 1 }, sub {

                $self->writeElement( 'bar', {}, 'text' );
                $self->writeElement( 'baz', {}, sub {
                    $self->writeElement( 'poo', {}, 'more_text' );
                } );

                $self->namespace('http://acme.com/serviceuri/2', sub {
                     $self->writeElement( 'foo', { id => 2 }, 'text' );
                });

                $self->namespace('', sub {
                     $self->writeElement( 'foo', { id => 3 }, 'text' );
                });
            });
        });
    });
}
